import json
from functools import wraps
from django.http import HttpResponseBadRequest, HttpResponse, HttpRequest, JsonResponse
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.http import require_POST
from django.conf import settings
from django.core.files.storage import default_storage
from django.urls import reverse


class HttpResponseUnAuth(HttpResponse):
    status_code = 401


def check_rest_framework_login(request):
    try:
        from rest_framework.settings import api_settings
        for auth_cls in api_settings.DEFAULT_AUTHENTICATION_CLASSES:
            auth = auth_cls()
            setattr(request, '_request', request)
            user = auth.authenticate(request)
            if user is not None:
                return user
    except Exception:
        pass


def check_login(view_func):

    @wraps(view_func)
    def wrapper_view(*args, **kwargs):
        should_auth = getattr(settings, 'MADMIN', {}).get('upload_auth', False)
        if should_auth:
            request = args[0] if len(args) else kwargs.get('request')
            user = request.user if request else None
            if not user or not user.is_authenticated:
                if not check_rest_framework_login(request):
                    return HttpResponseUnAuth()
        return view_func(*args, **kwargs)

    return wrapper_view


def get_file_path(hash, name):
    prefix = getattr(settings, 'MADMIN', {}).get('upload_path_prefix', 'madmin')
    keep_file_name = getattr(settings, 'MADMIN', {}).get('upload_keep_file_name', True)
    if keep_file_name:
        file_path = '{}/{}_{}'.format(prefix, hash, name)
    else:
        ext_parts = name.rsplit('.', 1)
        ext = ('.' + ext_parts[1]) if len(ext_parts) > 1 else ''
        file_path = '{}/{}{}'.format(prefix, hash, ext)
    return file_path


def get_request_data(request: HttpRequest):
    data = request.POST
    try:
        data = json.loads(request.body)
    except Exception:
        pass
    return data or {}


@check_login
@csrf_exempt
@require_POST
def upload(request: HttpRequest):
    data = get_request_data(request)
    hash = data.get('hash')
    file = request.FILES.get('file')
    if not bool(hash):
        return HttpResponseBadRequest('缺少输入参数hash')
    if not bool(file):
        return HttpResponseBadRequest('缺少输入参数file')

    file_path = get_file_path(hash, file.name)

    if not default_storage.exists(file_path):
        default_storage.save(file_path, file)

    file_url = default_storage.url(file_path)
    return JsonResponse({'file_url': file_url})


@check_login
@csrf_exempt
@require_POST
def check_upload(request: HttpRequest):
    data = get_request_data(request)
    hash = data.get('hash')
    name = data.get('name')
    if not bool(hash):
        return HttpResponseBadRequest('缺少输入参数hash')
    if not bool(name):
        return HttpResponseBadRequest('缺少输入参数name')

    file_path = get_file_path(hash, name)
    file_url = ''
    upload_url = ''
    if default_storage.exists(file_path):
        file_url = default_storage.url(file_path)
    else:
        upload_url = reverse(upload)
    return JsonResponse({'file_url': file_url, 'upload_url': upload_url})



from django.contrib.admin.options import ModelAdmin
from django.contrib.admin.filters import ListFilter


ModelAdmin.change_list_template = "admin/m_change_list.html"
ListFilter.template = "admin/m_filter.html"



from django import forms
from django.templatetags.static import static
from django.conf import settings
from django.utils.crypto import get_random_string
from pathlib import Path


def get_vue_js():
    js_dir = Path(__file__).resolve().parent.joinpath('static/madmin/js')
    for p in js_dir.iterdir():
        if not p.name.startswith('index-'):
            continue
        return 'madmin/js/' + p.name


VUE_JS = get_vue_js()

VUE_DEV_URL = 'http://127.0.0.1:5199/src/main.js'


class VueComponent(forms.Widget):
    template_name = "madmin/vue_component.html"

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        widget = context.get('widget')
        attrs = widget.setdefault('attrs', {})
        attrs['class'] = attrs.get('class', '') + ' flex flex-col'
        id = attrs.setdefault('id', 'id_{}'.format(get_random_string(8)))
        widget['id'] = id
        return context

    class Media:

        class MAdminJS:
            def __html__():
                dev = getattr(settings, 'MADMIN', {}).get('dev', False)
                js = VUE_DEV_URL if dev else static(VUE_JS)
                return '<script type="module" crossorigin src="{}"></script>'.format(js) if js else ''

        js = [MAdminJS]


class AsyncFileUpload(VueComponent):

    def get_context(self, name, value, attrs):
        prefix = getattr(settings, 'MADMIN', {}).get('upload_path_prefix', 'madmin')
        context = super().get_context(name, value, attrs)
        context.get('widget').update({
            'component': 'MFileUpload',
            'check_upload_url': '/{}/check_upload/'.format(prefix)
        })
        return context


class AsyncImageUpload(AsyncFileUpload):

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context.get('widget').update({'upload_type': 'image'})
        return context


class AsyncVideoUpload(AsyncFileUpload):

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        context.get('widget').update({'upload_type': 'video'})
        return context


class AsyncMultiFileUpload(VueComponent):

    def get_context(self, name, value, attrs):
        prefix = getattr(settings, 'MADMIN', {}).get('upload_path_prefix', 'madmin')
        attrs = attrs or {}
        attrs.update({'class': 'sm:!w-[486px] w-full'})
        context = super().get_context(name, value, attrs)
        context.get('widget').update({
            'component': 'MMultiFileUpload',
            'check_upload_url': '/{}/check_upload/'.format(prefix),
        })
        return context



import setuptools
from django_madmin import __version__ as version

with open('README.md', 'r', encoding='utf-8') as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-madmin",
    version=version,
    author="IT Expendables",
    author_email="ericalunson@gmail.com",
    description="django admin plus",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/IT-Expendables/django_madmin",
    packages=setuptools.find_packages(exclude=["demo", "demo.*"]),
    include_package_data=True,
    install_requires=['django>=4.0.0,<5.0.0'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 4.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System :: Systems Administration :: Authentication/Directory',
    ],
)



from django.conf import settings
from django.urls import path
from . import views


if settings.DEBUG:
    from django.utils.autoreload import file_changed
    file_changed.disconnect(dispatch_uid='template_loaders_file_changed')


prefix = getattr(settings, 'MADMIN', {}).get('upload_path_prefix', 'madmin')


madmin_urls = [
    path('{}/upload/'.format(prefix), views.upload, name="upload"),
    path('{}/check_upload/'.format(prefix), views.check_upload, name="check_upload"),
]
